/*
 * AbstractCompositeNumber.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

import java.math.*;

/**
 * Abstract base class for <code>CompositeNumber</code> implementations. This
 * implementation is based on an array. Concrete subclasses should initialize
 * the array upon construction.
 *
 * @version 1.1 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */

//----------------------------------------------------------------------------
// CHANGES SINCE 1.0a3:
//
// - Removed documentation where specified by an interface
//
//----------------------------------------------------------------------------

public abstract class AbstractCompositeNumber
   extends AbstractRealNumber
   implements CompositeNumber
{
   //=========================================================================
   // PROPERTY: elements
   //
   // - elements
   // - getElements
   // - getElementCount
   // - getElement
   //
   //=========================================================================

   /**
    * The elements of this operation.
    *
    * <p>Concrete subclasses must initialize this field upon construction.
    */
   protected RealNumber[] elements;

   public RealNumber[] getElements()
   {
      return (RealNumber[]) elements.clone();
   }

   public int getElementCount()
   {
      return elements.length;
   }

   public RealNumber getElement(int n)
      throws IndexOutOfBoundsException
   {
      return elements[n];
   }

}
