/*
 * CompositeNumber.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

/**
 * A composite real number. Examples of composite numbers include
 * sines, logarithms, powers, sums and products.
 *
 * @version 1.1 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */
public interface CompositeNumber
   extends RealNumber
{
   //=========================================================================
   // PROPERTY: operands
   //
   // - getElements
   // - getElementCount
   // - getElement
   //
   //=========================================================================

   /**
    * Returns the operands.
    *
    * @return
    *    the operands.
    */
   public RealNumber[] getElements();

   /**
    * Counts the number of operands in this <code>CompositeNumber</code>.
    *
    * @return
    *    the operand count.
    */
   public int getElementCount();

   /**
    * Returns the <em>n</em>th operand of this <code>CompositeNumber</code>.
    *
    * @param n
    *    the index of the operand.
    *
    * @return
    *    the <em>n</em>th operand.
    *
    * @exception IndexOutOfBoundsException
    *    if one of the following applies:
    *    <ol>
    *       <li><em>n</em> &lt;  0</li>
    *       <li><em>n</em> &gt;= <em>operandCount</em></li>
    *    </ol>
    */
   public RealNumber getElement(int n)
      throws IndexOutOfBoundsException;

}
