/*
 * Power.java
 *
 * Copyright (C) 1998  Utrecht University, Netherlands
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.coffee.math;

import java.math.*;

/**
 * A power, consisting of a base and an exponent.
 *
 * @version 1.1 199809.24
 * @author Ernst de Haan
 *
 * @since JUMP_1.0
 */
public class Power
   extends AbstractCompositeNumber
{
   //=========================================================================
   // CONSTRUCTOR
   //=========================================================================

   /**
    * Constructs a <code>Power</code> with the specified base and exponent.
    *
    * @param base the base for the power.
    * @param exponent the exponent for the power.
    * @exception IllegalArgumentException if either argument
    *    is <code>null</code>.
    */
   protected Power(RealNumber base,
                   RealNumber exponent)
      throws IllegalArgumentException
   {
      this.elements = new RealNumber[2];
      elements[0] = base;
      elements[1] = exponent;

      sign = determineSign(base, exponent);
   }


   //=========================================================================
   // FACTORY METHODS:
   //
   // - createInstance
   //
   //=========================================================================

   /**
    * Returns a <code>Power</code> with the specified operands.
    *
    * @param a the first operand.
    * @param b the second operand.
    * @return the <code>Power</code> instance, possibly newly constructed.
    * @exception IllegalArgumentException if either argument is
    *    <code>null</code>.
    */
   public static Power createInstance(RealNumber a, RealNumber b)
      throws IllegalArgumentException
   {
      return new Power(a, b);
   }


   //=========================================================================
   // DETERMINE SIGN OF SUM:
   //
   // - determineSign
   //
   //=========================================================================

   /**
    * Computes the sign of a power with the specified operands.
    *
    * @param operand1 the first operand.
    * @param operand2 the second operand.
    */
   protected static int determineSign(RealNumber a, RealNumber b)
   {
      //-----------------
      // PENDING: Do this
      //-----------------

      throw new RuntimeException("Unable to determine sign of power");
   }


   //=========================================================================
   // BASE AND EXPONENT:
   //
   // - getBase
   // - getExponent
   //
   //=========================================================================

   /**
    * Returns the base of this power.
    *
    * @return the base.
    */
   public RealNumber getBase()
   {
      return elements[0];
   }

   /**
    * Returns the exponent of this power.
    *
    * @return the exponent.
    */
   public RealNumber getExponent()
   {
      return elements[1];
   }


   //=========================================================================
   // PROPERTY: sign
   //
   // - sign
   // - getSign
   //
   //=========================================================================

   /**
    * The sign of this power. This field is set only once upon construction.
    */
   protected int sign;

   /**
    * Determines the sign of this number. This method returns -1, 0, or
    * 1 as this number is negative, zero, or positive.
    *
    * @return the sign of this number.
    */
   public int getSign()
   {
      return sign;
   }


   //=========================================================================
   // COMPARISON:
   //=========================================================================

   public int retryCompare(RealNumber n)
      throws IllegalArgumentException
   {
      //------------------------------
      // PENDING: How do we do this ?!
      //------------------------------

      throw new RuntimeException();
   }


   //=========================================================================
   // NUMERIC CONVERSION
   //
   // - toBigDecimal
   //
   //=========================================================================

   public BigDecimal toBigDecimal(int precision, int roundingMode)
      throws IllegalArgumentException
   {
      //------------------------
      // PENDING: Work this out.
      //------------------------

      throw new RuntimeException("Unable to translate Power to BigDecimal");
   }


   //=========================================================================
   // ROUNDING:
   //
   // - trunc
   //
   //=========================================================================

   public IntegerNumber trunc()
   {
      //-------------------------------------
      // PENDING: Reconsider this
      //          This is pretty safe for now
      //-------------------------------------

      return NumberCentral.valueOf(toBigDecimal(0)).trunc();
   }


   //=========================================================================
   // TEXTUAL CONVERSION:
   //
   // - toString
   //
   //=========================================================================

   public String toString()
   {
      StringBuffer buffer = new StringBuffer();

      if (elements[0] instanceof CompositeNumber)
      {
         buffer.append('(');
         buffer.append(elements[0]);
         buffer.append(")**");
      }
      else
      {
         buffer.append(elements[0]);
         buffer.append("**");
      }

      if (elements[1] instanceof CompositeNumber)
      {
         buffer.append('(');
         buffer.append(elements[1]);
         buffer.append(')');
      }
      else
      {
         buffer.append(elements[1]);
      }

      return buffer.toString();
   }

}
